from .core import add_command
from cmdcheatsheet.shared.models import Command, CommandArgument, CommandDetails


class AddCommand(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--add', '-a'],
            'Add new command to the list.',
            [CommandArgument('command'), CommandArgument('description')])

    def handler(self, args):
        command = Command(command=args[0], description=args[1])
        add_command(command)




from .display_available_configs import DisplayAvailableConfigs
from .display_config import DisplayConfig
from .remove_config import RemoveConfig
from .set_config import SetConfig
from .set_config_to_default import SetConfigToDefault
from .set_single_config_to_default import SetSingleConfigToDefault

config_actions = [
    DisplayAvailableConfigs(),
    DisplayConfig(),
    RemoveConfig(),
    SetConfig(),
    SetConfigToDefault(),
    SetSingleConfigToDefault()
]



from .add_alt_store import AddAltStore
from .delete_alt_store import DeleteAltStore
from .display_applied_alt_store_name import DisplayAppliedAltStoreName
from .display_available_alt_stores import DisplayAvailableAltStores
from .switch_to_alt_store import SwitchToAltStore
from .update_alt_store import UpdateAltStore

alt_store_actions = [
    AddAltStore(),
    DeleteAltStore(),
    DisplayAppliedAltStoreName(),
    DisplayAvailableAltStores(),
    SwitchToAltStore(),
    UpdateAltStore()
]



from .display import display_command_by_name 
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails


class GlobalFindCommand(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--global-find', '-gf'],
            'Global search.',
            [CommandArgument('command')])

    def handler(self, args):
        display_command_by_name(query=args[0], is_global=True)





from rich import print
from rich.console import Console
from rich.table import Table
from .core import *
from cmdcheatsheet.shared.display import *
from .messages import show_command_not_found


def display_command_by_name(query, display_index=False, is_global=False):
    commands_to_display = get_command_by_name(query, is_global)
    if commands_to_display:
        id_column_size = calc_id_column_size(commands_to_display)
        for command in commands_to_display:
            display_find_command(query, command,
                display_index, id_column_size, is_global)
    else:
        show_command_not_found()

def display_command_list(display_index=False):
    command_dict = group_commands_by_name()
    id_column_size = calc_id_column_size(get_commands())
    for value in command_dict.values():
        for item in value:
            display_command(item, display_index, id_column_size)

def display_find_command(query, command, display_index, id_column_length, is_global):
    index_cell = get_index_content(command.id, display_index, id_column_length)
    command_name = highlight_find_results(command.command, query)
    command_description = command.description
    if is_global:
        command_description = highlight_find_results(command.description, query, GREEN)
    print(f"{index_cell}[{BLUE}] {command_name}[{GREEN}] - {command_description}")

def display_command(command, display_index, id_column_length=5):
    index_cell = get_index_content(command.id, display_index, id_column_length)
    print(f"{index_cell}[{BLUE}] {command.command}[{GREEN}] - {command.description}")

def get_index_content(command_id, display_index, id_column_length):
    id = str(command_id) + ' '*(id_column_length - len(str(command_id)))
    return f"[{BLUE}]|[{YELLOW}] {id}[{BLUE}]  |" if display_index else ""

def calc_id_column_size(commands):
    command_ids = [c.id for c in commands]
    highest_command_id = max(command_ids) if command_ids else ""
    return len(str(highest_command_id))

def display_commands_table_view(query=''):
    all_commands = get_command_by_name(query, True) if query else get_commands()
    console = Console()
    table = Table(show_header=True, header_style=RED)
    table.add_column("Id")
    table.add_column("Command")
    table.add_column("Description")
    for c in all_commands:
        command = highlight_find_results(c.command, query)
        description = highlight_find_results(c.description, query, GREEN)
        table.add_row(
            f'[{YELLOW}]{str(c.id)}',
            f'[{BLUE}]{command}',
            f'[{GREEN}]{description}'
        )
    console.print(table)

def display_command_name_list(column_amount):
    commands = get_command_name_list()
    chunked_list = []
    chunk_size = column_amount if len(commands) >= column_amount else len(commands)
    if not chunk_size:
        print(f'[{RED}] No commands have been found yet.')
        return
    for i in range(0, len(commands), chunk_size):
        chunked_list.append(commands[i:i+chunk_size])
    console = Console()
    table = Table(show_header=False, header_style=RED)
    for i in range(chunk_size):
        table.add_column()
    for chunk in chunked_list:
        row = []
        for cn in range(0, chunk_size):
            row.append(f'[{GREEN}]{chunk[cn] if len(chunk) > cn else ""}')
        table.add_row(*row) 
    console.print(table)




from .display import display_command_by_name 
from cmdcheatsheet.shared.models import CommandArgument, CommandDetails


class DetailedGlobalFindCommand(CommandDetails):
    def __init__(self):
       super().__init__(
            ['--detailed-global-find', '-gfi'],
            'Search globally for a command and include all details, such as ids, etc.',
            [CommandArgument('command')])

    def handler(self, args):
        display_command_by_name(query=args[0],
            display_index=True, is_global=True)





from rich import print


BLUE = "turquoise2"
RED = "deep_pink2"
GREEN = "spring_green1"
YELLOW = "yellow"

def display_info(message):
    print(f"[{BLUE}]{message}")

def display_error(message):
    print(f"[{RED}]{message}")

def highlight_find_results(src, text_to_highlight, reset_color=BLUE):
    return f"{src}".replace(text_to_highlight,
        f"[{RED}]{text_to_highlight}[{reset_color}]")




#!/bin/python3
import sys

from .shared.display import display_error
from .general.core import get_command_by_name, is_help_action
from .config.core import setup_config
from .commands.core import setup_commands_store_config
from .commands.actions import command_actions
from .general.actions import general_actions
from .config.actions import config_actions
from .alt_store.actions import alt_store_actions


def main():
    setup_config()
    setup_commands_store_config()

    if len(sys.argv) == 1:
        sys.argv.append('-c')

    command = sys.argv[1]
    args = sys.argv[2:]

    try:
        program_actions = [
            *command_actions,
            *general_actions,
            *config_actions,
            *alt_store_actions
        ]

        command_to_invoke = get_command_by_name(program_actions, command)
        if command_to_invoke:
            
            if is_help_action(command):
                args.append(program_actions)

            command_to_invoke.handle(args)
        else:
            display_error("Please provide the valid command name.")

    except Exception as e:
        display_error(e)

if __name__ == '__main__':
    main()
